// PLAY3S.C
// This program uses the non-DMA play3s function. Memory is 
// allocated for the sound data. A file name TEST1.V3S is opened
// and its contents read into a buffer. Finally, the allocated
// memory is freed.

#include <stdio.h>
#include <conio.h>
#include "cvxdigi.h"
#include "cvxutil.h"

#define _BUFFER_SIZE     0x8000
#define _SOUND_FILE      "TEST1.V3S"
#define _TREBLE_FLAG     _FALSE
#define _NOISE_FLAG      _FALSE

VOID main( VOID )
{
   HANDLE    fileHandle;
   LPSTR     playBuffer;
   LONG      bytesAvailable;
   LONG      bytesRead;

   // Allocate memory for playback buffer.
   playBuffer = cvxBufferAlloc( ( LONG )_BUFFER_SIZE, &bytesAvailable );

   if( playBuffer == _NULL )
      printf( "ERROR : Cannot Allocate Memory!!\n" );
   else
   {
      // Open the file containing sound data.
      if( cvxFileOpen( _SOUND_FILE, _OPEN_R_ONLY, &fileHandle ) )
         printf( "ERROR : %s not found.\n", _SOUND_FILE );
      else
      {
         // Read sound data from file.
         cvxFileRead( fileHandle, playBuffer,
                      ( LONG )_BUFFER_SIZE, &bytesRead );
      
         // Prompt user to hit a key to begin playback.
         printf( "Hit any key to begin playback.\n" );
         
         // Wait for keystroke.
         while( !kbhit() );
   
         // Clear keystroke(s) from keyboard buffer.
         while( kbhit() )
            getch();
   
         // Notify user that playback has begun.
         printf( "Playing sound file ..." );
   
         // Playback a file of length bytesRead.
         play3s( playBuffer, ( LONG )bytesRead, _CVX_RATE_DEFAULT,
                 _CVX_VM0, _TREBLE_FLAG, _NOISE_FLAG );
   
         // Notify user that playback has completed.
         printf( " complete.\n" );
   
         // Close file containing sound data.
         cvxFileClose( fileHandle );
      }

      // Free memory used by buffer.
      if( cvxBufferFree( playBuffer ) )
         printf( "ERROR : Cannot Free Memory!!\n" );
   }
}



